@@ -3,8 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
- * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2025 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2025 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -89,7 +89,7 @@ namespace Orthanc
   }
 
 
-  DicomServer::DicomServer() : 
+  DicomServer::DicomServer() :
     pimpl_(new PImpl),
     checkCalledAet_(true),
     aet_("ANY-SCP"),
@@ -108,7 +108,8 @@ namespace Orthanc
     useDicomTls_(false),
     maximumPduLength_(ASC_DEFAULTMAXPDU),
     remoteCertificateRequired_(true),
-    minimumTlsVersion_(0)
+    minimumTlsVersion_(0),
+    metricsRegistry_(NULL)
   {
   }
 
@@ -432,7 +433,15 @@ namespace Orthanc
 
     CLOG(INFO, DICOM) << "The embedded DICOM server will use " << threadsCount_ << " threads";
 
-    pimpl_->workers_.reset(new RunnableWorkersPool(threadsCount_, "DICOM-"));
+    if (metricsRegistry_ == NULL)
+    {
+      pimpl_->workers_.reset(new RunnableWorkersPool(threadsCount_, "DICOM-"));
+    }
+    else
+    {
+      pimpl_->workers_.reset(new RunnableWorkersPool(threadsCount_, "DICOM-", *metricsRegistry_, "orthanc_available_dicom_threads"));
+    }
+
     pimpl_->thread_ = boost::thread(ServerThread, this, maximumPduLength_, useDicomTls_);
   }
 
@@ -620,4 +629,9 @@ namespace Orthanc
     threadsCount_ = threads;
   }
 
+  void DicomServer::SetMetricsRegistry(MetricsRegistry& registry)
+  {
+    Stop();
+    metricsRegistry_ = &registry;
+  }
 }
