@@ -176,7 +176,7 @@ namespace Orthanc
         LOG(INFO) << "Created attachment \"" << uuid << "\" (" << timer.GetHumanTransferSpeed(true, size) << ")";
         return;
       }
-      catch (OrthancException& e)
+      catch (OrthancException&)
       {
         if (retryCount >= maxRetryCount)
         {
@@ -187,8 +187,8 @@ namespace Orthanc
   }
 
 
-  IMemoryBuffer* FilesystemStorage::Read(const std::string& uuid,
-                                         FileContentType type)
+  IMemoryBuffer* FilesystemStorage::ReadWhole(const std::string& uuid,
+                                              FileContentType type)
   {
     Toolbox::ElapsedTimer timer;
     LOG(INFO) << "Reading attachment \"" << uuid << "\" of \"" << GetDescriptionInternal(type) 
@@ -221,12 +221,6 @@ namespace Orthanc
   }
 
 
-  bool FilesystemStorage::HasReadRange() const
-  {
-    return true;
-  }
-
-
   uintmax_t FilesystemStorage::GetSize(const std::string& uuid) const
   {
     boost::filesystem::path path = GetPath(uuid);
@@ -354,7 +348,7 @@ namespace Orthanc
                                const std::string& uuid,
                                FileContentType type)
   {
-    std::unique_ptr<IMemoryBuffer> buffer(Read(uuid, type));
+    std::unique_ptr<IMemoryBuffer> buffer(ReadWhole(uuid, type));
     buffer->MoveToString(content);
   }
 #endif
