@@ -348,6 +348,7 @@ namespace Orthanc
     bool                  isJQueryUploadChunk_;
     std::string           jqueryUploadFileName_;
     size_t                jqueryUploadFileSize_;
+    const std::string&    authenticationPayload_;
 
     void HandleInternal(const MultipartStreamReader::HttpHeaders& headers,
                         const void* part,
@@ -358,7 +359,7 @@ namespace Orthanc
       HttpToolbox::GetArguments getArguments;
       
       if (!handler_.Handle(fakeOutput, RequestOrigin_RestApi, remoteIp_.c_str(), username_.c_str(), 
-                           HttpMethod_Post, uri_, headers, getArguments, part, size))
+                           HttpMethod_Post, uri_, headers, getArguments, part, size, authenticationPayload_))
       {
         throw OrthancException(ErrorCode_UnknownResource);
       }
@@ -370,14 +371,16 @@ namespace Orthanc
                              const std::string& remoteIp,
                              const std::string& username,
                              const UriComponents& uri,
-                             const MultipartStreamReader::HttpHeaders& headers) :
+                             const MultipartStreamReader::HttpHeaders& headers,
+                             const std::string& authenticationPayload) :
       handler_(handler),
       chunkStore_(chunkStore),
       remoteIp_(remoteIp),
       username_(username),
       uri_(uri),
       isJQueryUploadChunk_(false),
-      jqueryUploadFileSize_(0)  // Dummy initialization
+      jqueryUploadFileSize_(0),  // Dummy initialization
+      authenticationPayload_(authenticationPayload)
     {
       typedef HttpToolbox::Arguments::const_iterator Iterator;
       
@@ -470,9 +473,10 @@ namespace Orthanc
                                             const UriComponents& uri,
                                             const std::map<std::string, std::string>& headers,
                                             const std::string& body,
-                                            const std::string& boundary)
+                                            const std::string& boundary,
+                                            const std::string& authenticationPayload)
   {
-    MultipartFormDataHandler handler(GetHandler(), pimpl_->chunkStore_, remoteIp, username, uri, headers);
+    MultipartFormDataHandler handler(GetHandler(), pimpl_->chunkStore_, remoteIp, username, uri, headers, authenticationPayload);
           
     MultipartStreamReader reader(boundary);
     reader.SetHandler(handler);
@@ -621,7 +625,7 @@ namespace Orthanc
   
   enum AccessMode
   {
-    AccessMode_Forbidden,
+    AccessMode_Unauthorized,
     AccessMode_AuthorizationToken,
     AccessMode_RegisteredUser
   };
@@ -657,7 +661,7 @@ namespace Orthanc
       }
     }
 
-    return AccessMode_Forbidden;
+    return AccessMode_Unauthorized;
   }
 
 
@@ -1146,13 +1150,52 @@ namespace Orthanc
   } 
 #endif /* ORTHANC_ENABLE_PUGIXML == 1 */
 
-  
+
+  std::string HttpServer::GetRelativePathToRoot(const std::string& uri)
+  {
+    if (uri.empty())
+    {
+      throw OrthancException(ErrorCode_ParameterOutOfRange);
+    }
+
+    if (uri == "/")
+    {
+      return "./";
+    }
+
+    std::string path;
+
+    if (uri[uri.size() - 1] == '/')
+    {
+      path = "../";
+    }
+    else
+    {
+      path = "./";
+    }
+
+    UriComponents components;
+    Toolbox::SplitUriComponents(components, uri);
+
+    for (size_t i = 1; i < components.size(); i++)
+    {
+      path += "../";
+    }
+
+    return path;
+  }
+
+
   static void InternalCallback(HttpOutput& output /* out */,
                                HttpMethod& method /* out */,
                                HttpServer& server,
                                struct mg_connection *connection,
                                const struct mg_request_info *request)
   {
+    server.UpdateCurrentThreadName();
+
+    std::unique_ptr<MetricsRegistry::AvailableResourcesDecounter> counter(server.CreateAvailableHttpThreadsDecounter());
+
     bool localhost;
 
 #if ORTHANC_ENABLE_MONGOOSE == 1
@@ -1200,18 +1243,10 @@ namespace Orthanc
       HttpToolbox::ParseGetArguments(argumentsGET, request->query_string);
     }
 
-    
+
     AccessMode accessMode = IsAccessGranted(server, headers);
 
-    // Authenticate this connection
-    if (server.IsAuthenticationEnabled() && 
-        accessMode == AccessMode_Forbidden)
-    {
-      output.SendUnauthorized(server.GetRealm());   // 401 error
-      return;
-    }
 
-    
 #if ORTHANC_ENABLE_MONGOOSE == 1
     // Apply the filter, if it is installed
     char remoteIp[24];
@@ -1235,6 +1270,55 @@ namespace Orthanc
       requestUri = "";
     }
       
+
+    const IIncomingHttpRequestFilter *filter = server.GetIncomingHttpRequestFilter();
+
+    // Authenticate this connection
+    std::string authenticationPayload;
+    std::string redirection;
+    IIncomingHttpRequestFilter::AuthenticationStatus status;
+
+    if (filter == NULL)
+    {
+      status = IIncomingHttpRequestFilter::AuthenticationStatus_BuiltIn;
+    }
+    else
+    {
+      status = filter->CheckAuthentication(authenticationPayload, redirection, requestUri, remoteIp, headers, argumentsGET);
+    }
+
+    switch (status)
+    {
+      case IIncomingHttpRequestFilter::AuthenticationStatus_BuiltIn:
+        // This was the only behavior available in Orthanc <= 1.12.8
+        if (server.IsAuthenticationEnabled() &&
+            accessMode == AccessMode_Unauthorized)
+        {
+          output.SendUnauthorized(server.GetRealm());   // 401 error
+          return;
+        }
+        break;
+
+      case IIncomingHttpRequestFilter::AuthenticationStatus_Granted:
+        break;
+
+      case IIncomingHttpRequestFilter::AuthenticationStatus_Redirect:
+        output.Redirect(Toolbox::JoinUri(HttpServer::GetRelativePathToRoot(requestUri), redirection));
+        return;
+
+      case IIncomingHttpRequestFilter::AuthenticationStatus_Unauthorized:
+        output.SendStatus(HttpStatus_401_Unauthorized);
+        return;
+
+      case IIncomingHttpRequestFilter::AuthenticationStatus_Forbidden:
+        output.SendStatus(HttpStatus_403_Forbidden);
+        return;
+
+      default:
+        throw OrthancException(ErrorCode_ParameterOutOfRange);
+    }
+
+
     // Decompose the URI into its components
     UriComponents uri;
     try
@@ -1257,10 +1341,12 @@ namespace Orthanc
     
     HttpMethod filterMethod;
 
-    
+    std::unique_ptr<Toolbox::ApiElapsedTimeLogger> apiLogTimer; // to log the time spent in the API call
+
     if (ExtractMethod(method, request, headers, argumentsGET))
     {
-      CLOG(INFO, HTTP) << EnumerationToString(method) << " " << Toolbox::FlattenUri(uri);
+      apiLogTimer.reset(new Toolbox::ApiElapsedTimeLogger(std::string(EnumerationToString(method)) + " " + Toolbox::FlattenUri(uri)));
+      
       filterMethod = method;
     }
 #if ORTHANC_ENABLE_PUGIXML == 1
@@ -1268,8 +1354,7 @@ namespace Orthanc
              !strcmp(request->request_method, "PROPFIND") ||
              !strcmp(request->request_method, "HEAD"))
     {
-      CLOG(INFO, HTTP) << "Incoming read-only WebDAV request: "
-                       << request->request_method << " " << requestUri;
+      apiLogTimer.reset(new Toolbox::ApiElapsedTimeLogger(std::string("Incoming read-only WebDAV request: ") + request->request_method + " " + requestUri));
       filterMethod = HttpMethod_Get;
       isWebDav = true;
     }
@@ -1278,8 +1363,7 @@ namespace Orthanc
              !strcmp(request->request_method, "UNLOCK") ||
              !strcmp(request->request_method, "MKCOL"))
     {
-      CLOG(INFO, HTTP) << "Incoming read-write WebDAV request: "
-                       << request->request_method << " " << requestUri;
+      apiLogTimer.reset(new Toolbox::ApiElapsedTimeLogger(std::string("Incoming read-write WebDAV request: ") + request->request_method + " " + requestUri));
       filterMethod = HttpMethod_Put;
       isWebDav = true;
     }
@@ -1300,10 +1384,9 @@ namespace Orthanc
       // filter. In the case of an authorization bearer token, grant
       // full access to the API.
 
-      assert(accessMode == AccessMode_Forbidden ||  // Could be the case if "!server.IsAuthenticationEnabled()"
+      assert(accessMode == AccessMode_Unauthorized ||  // Could be the case if "!server.IsAuthenticationEnabled()"
              accessMode == AccessMode_RegisteredUser);
       
-      IIncomingHttpRequestFilter *filter = server.GetIncomingHttpRequestFilter();
       if (filter != NULL &&
           !filter->IsAllowed(filterMethod, requestUri, remoteIp,
                              username.c_str(), headers, argumentsGET))
@@ -1339,7 +1422,7 @@ namespace Orthanc
     if (method == HttpMethod_Post ||
         method == HttpMethod_Put)
     {
-      PostDataStatus status;
+      PostDataStatus postStatus;
 
       bool isMultipartForm = false;
 
@@ -1356,10 +1439,10 @@ namespace Orthanc
          **/
         isMultipartForm = true;
 
-        status = ReadBodyToString(body, connection, headers);
-        if (status == PostDataStatus_Success)
+        postStatus = ReadBodyToString(body, connection, headers);
+        if (postStatus == PostDataStatus_Success)
         {
-          server.ProcessMultipartFormData(remoteIp, username, uri, headers, body, boundary);
+          server.ProcessMultipartFormData(remoteIp, username, uri, headers, body, boundary, authenticationPayload);
           output.SendStatus(HttpStatus_200_Ok);
           return;
         }
@@ -1372,7 +1455,7 @@ namespace Orthanc
         if (server.HasHandler())
         {
           found = server.GetHandler().CreateChunkedRequestReader
-            (stream, RequestOrigin_RestApi, remoteIp, username.c_str(), method, uri, headers);
+            (stream, RequestOrigin_RestApi, remoteIp, username.c_str(), method, uri, headers, authenticationPayload);
         }
         
         if (found)
@@ -1382,20 +1465,20 @@ namespace Orthanc
             throw OrthancException(ErrorCode_InternalError);
           }
 
-          status = ReadBodyToStream(*stream, connection, headers);
+          postStatus = ReadBodyToStream(*stream, connection, headers);
 
-          if (status == PostDataStatus_Success)
+          if (postStatus == PostDataStatus_Success)
           {
             stream->Execute(output);
           }
         }
         else
         {
-          status = ReadBodyToString(body, connection, headers);
+          postStatus = ReadBodyToString(body, connection, headers);
         }
       }
 
-      switch (status)
+      switch (postStatus)
       {
         case PostDataStatus_NoLength:
           output.SendStatus(HttpStatus_411_LengthRequired);
@@ -1421,7 +1504,7 @@ namespace Orthanc
         server.HasHandler())
     {
       found = server.GetHandler().Handle(output, RequestOrigin_RestApi, remoteIp, username.c_str(), 
-                                         method, uri, headers, argumentsGET, body.c_str(), body.size());
+                                         method, uri, headers, argumentsGET, body.c_str(), body.size(), authenticationPayload);
     }
 
     if (!found)
@@ -1534,8 +1617,6 @@ namespace Orthanc
     }
   }
 
-  static uint16_t threadCounter = 0;
-
 #if MONGOOSE_USE_CALLBACKS == 0
   static void* Callback(enum mg_event event,
                         struct mg_connection *connection,
@@ -1558,12 +1639,6 @@ namespace Orthanc
   static int Callback(struct mg_connection *connection)
   {
     const struct mg_request_info *request = mg_get_request_info(connection);
-
-    if (!Logging::HasCurrentThreadName())
-    {
-      Logging::SetCurrentThreadName(std::string("HTTP-") + boost::lexical_cast<std::string>(threadCounter++));
-    }
-
     ProtectedCallback(connection, request);
 
     return 1;  // Do not let Mongoose handle the request by itself
@@ -1601,7 +1676,8 @@ namespace Orthanc
     realm_(ORTHANC_REALM),
     threadsCount_(50),  // Default value in mongoose/civetweb
     tcpNoDelay_(true),
-    requestTimeout_(30)  // Default value in mongoose/civetweb (30 seconds)
+    requestTimeout_(30),  // Default value in mongoose/civetweb (30 seconds)
+    threadCounter_(0)
   {
 #if ORTHANC_ENABLE_MONGOOSE == 1
     CLOG(INFO, HTTP) << "This Orthanc server uses Mongoose as its embedded HTTP server";
@@ -1651,7 +1727,7 @@ namespace Orthanc
   void HttpServer::Start()
   {
     // reset thread counter used to generate HTTP thread names.
-    threadCounter = 0;
+    threadCounter_ = 0;
 
 #if ORTHANC_ENABLE_MONGOOSE == 1
     CLOG(INFO, HTTP) << "Starting embedded Web server using Mongoose";
@@ -2119,6 +2195,11 @@ namespace Orthanc
     Stop();
     threadsCount_ = threads;
 
+    if (availableHttpThreadsMetrics_.get() != NULL)
+    {
+      availableHttpThreadsMetrics_->SetInitialValue(threadsCount_);
+    }
+
     CLOG(INFO, HTTP) << "The embedded HTTP server will use " << threads << " threads";
   }
 
@@ -2204,4 +2285,39 @@ namespace Orthanc
     }
   }
 #endif
+
+
+  void HttpServer::SetMetricsRegistry(MetricsRegistry& metricsRegistry)
+  {
+    Stop();
+    availableHttpThreadsMetrics_.reset(new MetricsRegistry::SharedMetrics(
+                                         metricsRegistry,
+                                         "orthanc_available_http_threads_count",
+                                         MetricsUpdatePolicy_MinOver10Seconds));
+    availableHttpThreadsMetrics_->SetInitialValue(threadsCount_);
+  }
+
+
+  MetricsRegistry::AvailableResourcesDecounter* HttpServer::CreateAvailableHttpThreadsDecounter()
+  {
+    // NB: "availableHttpThreadsMetrics_" is protected by the mutex in "mg_stop()"
+    if (availableHttpThreadsMetrics_.get() != NULL)
+    {
+      return new MetricsRegistry::AvailableResourcesDecounter(*availableHttpThreadsMetrics_);
+    }
+    else
+    {
+      return NULL;
+    }
+  }
+
+
+  void HttpServer::UpdateCurrentThreadName()
+  {
+    if (!Logging::HasCurrentThreadName())
+    {
+      boost::mutex::scoped_lock lock(threadCounterMutex_);
+      Logging::SetCurrentThreadName(std::string("HTTP-") + boost::lexical_cast<std::string>(threadCounter_++));
+    }
+  }
 }
