@@ -3,8 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
- * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2025 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2025 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -106,6 +106,9 @@ namespace Orthanc
       SetIntegerValue(name, value, MetricsUpdatePolicy_Directly);
     }
     
+    void SetInitialValue(const std::string& name,
+                         int64_t value);
+
     void IncrementIntegerValue(const std::string& name,
                                int64_t delta);
 
@@ -131,6 +134,8 @@ namespace Orthanc
                     MetricsUpdatePolicy policy);
 
       void Add(int64_t delta);
+
+      void SetInitialValue(int64_t value);
     };
 
 
@@ -146,6 +151,17 @@ namespace Orthanc
     };
 
 
+    class ORTHANC_PUBLIC AvailableResourcesDecounter : public boost::noncopyable
+    {
+    private:
+      SharedMetrics&   metrics_;
+
+    public:
+      explicit AvailableResourcesDecounter(SharedMetrics& metrics);
+
+      ~AvailableResourcesDecounter();
+    };
+
     class ORTHANC_PUBLIC Timer : public boost::noncopyable
     {
     private:
