@@ -37,8 +37,10 @@
 #  error Cannot access the version of JsonCpp
 #endif
 
-#if !defined(ORTHANC_ENABLE_ICU)
+#if (ORTHANC_ENABLE_LOCALE == 1) && (BOOST_LOCALE_WITH_ICU == 1)
 #  define ORTHANC_ENABLE_ICU 1
+#else
+#  define ORTHANC_ENABLE_ICU 0
 #endif
 
 
@@ -64,6 +66,7 @@
 #include <boost/algorithm/string/join.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/regex.hpp>
+#include <cassert>
 
 #if BOOST_VERSION >= 106600
 #  include <boost/uuid/detail/sha1.hpp>
@@ -147,21 +150,22 @@ extern "C"
 }
 
 
-#if defined(ORTHANC_STATIC_ICU)
+#if ORTHANC_ENABLE_ICU == 1
 
-#  if (ORTHANC_STATIC_ICU == 1) && (ORTHANC_ENABLE_ICU == 1)
+#  if ORTHANC_STATIC_ICU == 1
 #    if !defined(ORTHANC_FRAMEWORK_INCLUDE_RESOURCES) || (ORTHANC_FRAMEWORK_INCLUDE_RESOURCES == 1)
 #      include <OrthancFrameworkResources.h>
 #    endif
+#    include "Compression/GzipCompressor.h"
 #  endif
 
-#  if (ORTHANC_STATIC_ICU == 1 && ORTHANC_ENABLE_LOCALE == 1)
-#    include <unicode/udata.h>
-#    include <unicode/uloc.h>
-#    include "Compression/GzipCompressor.h"
+#  include <unicode/udata.h>
+#  include <unicode/uloc.h>
+#  include <unicode/uclean.h>
 
 static std::string  globalIcuData_;
 
+#  if ORTHANC_STATIC_ICU == 1
 extern "C"
 {
   // This is dummy content for the "icudt58_dat" (resp. "icudt63_dat")
@@ -169,15 +173,18 @@ extern "C"
   // (resp. "icudt63l_dat.c") file that contains a huge C array. In
   // Orthanc, this array is compressed using gzip and attached as a
   // resource, then uncompressed during the launch of Orthanc by
-  // static function "InitializeIcu()".
+  // static function "InitializeIcu()". WARNING: Do NOT do this if
+  // dynamically linking against libicu!
   struct
   {
     double bogus;
     uint8_t *bytes;
   } U_ICUDATA_ENTRY_POINT = { 0.0, NULL };
 }
+#  endif
+
+#  if defined(__LSB_VERSION__)
 
-#    if defined(__LSB_VERSION__)
 extern "C"
 {
   /**
@@ -192,9 +199,9 @@ extern "C"
    **/
   char *tzname[2] = { (char *) "GMT", (char *) "GMT" };
 }
-#    endif
 
 #  endif
+
 #endif
  
 
@@ -207,6 +214,112 @@ extern "C"
 
 namespace Orthanc
 {
+#if ORTHANC_ENABLE_MD5 == 1
+  static char GetHexadecimalCharacter(uint8_t value)
+  {
+    assert(value < 16);
+
+    if (value < 10)
+    {
+      return value + '0';
+    }
+    else
+    {
+      return (value - 10) + 'a';
+    }
+  }
+
+
+  struct Toolbox::MD5Context::PImpl
+  {
+    md5_state_s  state_;
+    bool         done_;
+
+    PImpl() :
+      done_(false)
+    {
+      md5_init(&state_);
+    }
+  };
+
+
+  Toolbox::MD5Context::MD5Context() :
+    pimpl_(new PImpl)
+  {
+  }
+
+
+  void Toolbox::MD5Context::Append(const void* data,
+                                   size_t size)
+  {
+    static const size_t MAX_SIZE = 128 * 1024 * 1024;
+
+    if (pimpl_->done_)
+    {
+      throw OrthancException(ErrorCode_BadSequenceOfCalls);
+    }
+
+    const uint8_t *p = reinterpret_cast<const uint8_t*>(data);
+
+    while (size > 0)
+    {
+      /**
+       * The built-in implementation of MD5 requires that "size" can
+       * be casted to "int", so we feed it by chunks of maximum
+       * 128MB. This fixes an incorrect behavior in Orthanc <= 1.12.7.
+       **/
+
+      int chunkSize;
+      if (size > MAX_SIZE)
+      {
+        chunkSize = static_cast<int>(MAX_SIZE);
+      }
+      else
+      {
+        chunkSize = static_cast<int>(size);
+      }
+
+      md5_append(&pimpl_->state_, reinterpret_cast<const md5_byte_t*>(p), chunkSize);
+
+      p += chunkSize;
+
+      assert(static_cast<size_t>(chunkSize) <= size);
+      size -= chunkSize;
+    }
+  }
+
+
+  void Toolbox::MD5Context::Append(const std::string& source)
+  {
+    if (source.size() > 0)
+    {
+      Append(source.c_str(), source.size());
+    }
+  }
+
+
+  void Toolbox::MD5Context::Export(std::string& target)
+  {
+    if (pimpl_->done_)
+    {
+      throw OrthancException(ErrorCode_BadSequenceOfCalls);
+    }
+
+    pimpl_->done_ = true;
+
+    md5_byte_t actualHash[16];
+    md5_finish(&pimpl_->state_, actualHash);
+
+    target.resize(32);
+    for (unsigned int i = 0; i < 16; i++)
+    {
+      target[2 * i] = GetHexadecimalCharacter(static_cast<uint8_t>(actualHash[i] / 16));
+      target[2 * i + 1] = GetHexadecimalCharacter(static_cast<uint8_t>(actualHash[i] % 16));
+    }
+  }
+#endif  /* ORTHANC_ENABLE_MD5 */
+
+
   void Toolbox::LinesIterator::FindEndOfLine()
   {
     lineEnd_ = lineStart_;
@@ -444,21 +557,6 @@ namespace Orthanc
 
 
 #if ORTHANC_ENABLE_MD5 == 1
-  static char GetHexadecimalCharacter(uint8_t value)
-  {
-    assert(value < 16);
-
-    if (value < 10)
-    {
-      return value + '0';
-    }
-    else
-    {
-      return (value - 10) + 'a';
-    }
-  }
-
-
   void Toolbox::ComputeMD5(std::string& result,
                            const std::string& data)
   {
@@ -477,25 +575,9 @@ namespace Orthanc
                            const void* data,
                            size_t size)
   {
-    md5_state_s state;
-    md5_init(&state);
-
-    if (size > 0)
-    {
-      md5_append(&state, 
-                 reinterpret_cast<const md5_byte_t*>(data), 
-                 static_cast<int>(size));
-    }
-
-    md5_byte_t actualHash[16];
-    md5_finish(&state, actualHash);
-
-    result.resize(32);
-    for (unsigned int i = 0; i < 16; i++)
-    {
-      result[2 * i] = GetHexadecimalCharacter(static_cast<uint8_t>(actualHash[i] / 16));
-      result[2 * i + 1] = GetHexadecimalCharacter(static_cast<uint8_t>(actualHash[i] % 16));
-    }
+    MD5Context context;
+    context.Append(data, size);
+    context.Export(result);
   }
 
   void Toolbox::ComputeMD5(std::string& result,
@@ -622,7 +704,7 @@ namespace Orthanc
         return "GB18030";
 
       case Encoding_Thai:
-#if BOOST_LOCALE_WITH_ICU == 1
+#if ORTHANC_ENABLE_ICU == 1
         return "tis620.2533";
 #else
         return "TIS620.2533-0";
@@ -648,7 +730,8 @@ namespace Orthanc
   // http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.12.html#sect_C.12.1.1.2
   std::string Toolbox::ConvertToUtf8(const std::string& source,
                                      Encoding sourceEncoding,
-                                     bool hasCodeExtensions)
+                                     bool hasCodeExtensions,
+                                     bool skipBackslashes)
   {
 #if ORTHANC_STATIC_ICU == 1
 #  if ORTHANC_ENABLE_ICU == 0
@@ -684,7 +767,26 @@ namespace Orthanc
         else
         {
           const char* encoding = GetBoostLocaleEncoding(sourceEncoding);
-          s = boost::locale::conv::to_utf<char>(source, encoding, boost::locale::conv::skip);
+
+          if (skipBackslashes)
+          {
+            /**
+             * This is to deal with the fact that in Japanese coding
+             * (ISO_IR 13), backslashes will be converted to the Yen
+             * character.
+             **/
+            std::vector<std::string> tokens;
+            TokenizeString(tokens, source, '\\');
+            for (size_t i = 0; i < tokens.size(); i++)
+            {
+              tokens[i] = boost::locale::conv::to_utf<char>(tokens[i], encoding, boost::locale::conv::skip);
+            }
+            JoinStrings(s, tokens, "\\");
+          }
+          else
+          {
+            s = boost::locale::conv::to_utf<char>(source, encoding, boost::locale::conv::skip);
+          }
         }
 
         if (hasCodeExtensions)
@@ -754,6 +856,50 @@ namespace Orthanc
 #endif
 
 
+#if ORTHANC_ENABLE_LOCALE == 1
+  std::string Toolbox::ConvertDicomStringToUtf8(const std::string& source,
+                                                Encoding sourceEncoding,
+                                                bool hasCodeExtensions,
+                                                ValueRepresentation vr)
+  {
+    /**
+     * This method was added in Orthanc 1.12.9, as a consequence of:
+     * https://discourse.orthanc-server.org/t/issue-with-special-characters-when-scans-where-uploaded-with-specificcharacterset-dicom-tag-value-as-iso-ir-13/5962
+     *
+     * From the DICOM standard: "Two character codes of the
+     * single-byte character sets invoked in the GL area of the code
+     * table, 02/00 and 05/12, have special significance in the DICOM
+     * Standard. The character SPACE, represented by bit combination
+     * 02/00, shall be used for the padding of Data Element Values
+     * that are character strings. The Graphic Character represented
+     * by the bit combination 05/12, "\" (BACKSLASH) (reverse solidus)
+     * in the repertoire ISO-IR 6, shall only be used in character
+     * strings with Value Representations of UT, ST and LT (see
+     * Section 6.2). Otherwise the character code 05/12 is used as a
+     * separator for multi-valued Data Elements (see Section
+     * 6.4). [...] When the Value of Specific Character Set
+     * (0008,0005) is either "ISO_IR 13" or "ISO 2022 IR 13", the
+     * graphic character represented by the bit combination 05/12 is a
+     * "¥" (YEN SIGN) in the character set of ISO-IR 14."
+     * https://www.dicomstandard.org/standards/view/data-structures-and-encoding
+     *
+     * This description implies that if "sourceEncoding" (which is
+     * derived from the value of the DICOM Specific Character Set)
+     * corresponds "ISO_IR 13" or "ISO 2022 IR 13", AND if the value
+     * representation is *not* UT, ST, or LT, then backslashes should
+     * be ignored during the conversion to UTF-8.
+     **/
+
+    const bool skipBackslashes = (sourceEncoding == Encoding_Japanese &&
+                                  vr != ValueRepresentation_UnlimitedText &&  // UT
+                                  vr != ValueRepresentation_ShortText &&      // ST
+                                  vr != ValueRepresentation_LongText);        // LT
+
+    return ConvertToUtf8(source, sourceEncoding, hasCodeExtensions, skipBackslashes);
+  }
+#endif
+
+
   static bool IsAsciiCharacter(uint8_t c)
   {
     return (c != 0 &&
@@ -1406,10 +1552,25 @@ namespace Orthanc
             c == '-' ||
             c == '_' ||
             c == '.' ||
-            c == '~' ||
-            c == '/');
+            c == '~');
   }
 
+  // in this version, each path token is uri encoded separately and then all parts are joined with "/"
+  void Toolbox::UriEncode(std::string& target,
+                          const std::vector<std::string>& pathTokens)
+  {
+    std::vector<std::string> uriEncodedPathTokens;
+    for (std::vector<std::string>::const_iterator it = pathTokens.begin(); it != pathTokens.end(); ++it)
+    {
+      std::string encodedPathToken;
+      Toolbox::UriEncode(encodedPathToken, *it);
+      uriEncodedPathTokens.push_back(encodedPathToken);
+    }
+
+    Toolbox::JoinStrings(target, uriEncodedPathTokens, "/");
+  }
+
+
   void Toolbox::UriEncode(std::string& target,
                           const std::string& source)
   {
@@ -1696,15 +1857,24 @@ namespace Orthanc
         // TODO - The data table must be swapped (uint16_t)
         throw OrthancException(ErrorCode_NotImplemented);
       }
-
-      // "First-use of ICU from a single thread before the
-      // multi-threaded use of ICU begins", to make sure everything is
-      // properly initialized (should not be mandatory in our
-      // case). We let boost handle calls to "u_init()" and "u_cleanup()".
-      // http://userguide.icu-project.org/design#TOC-ICU-Initialization-and-Termination
-      uloc_getDefault();
     }
 #endif
+
+#if (ORTHANC_ENABLE_ICU == 1)
+    UErrorCode status = U_ZERO_ERROR;
+    u_init(&status);
+
+    if (U_FAILURE(status))
+    {
+      throw OrthancException(ErrorCode_InternalError, "Cannot initialize ICU: " + std::string(u_errorName(status)));
+    }
+
+    // "First-use of ICU from a single thread before the
+    // multi-threaded use of ICU begins", to make sure everything is
+    // properly initialized (should not be mandatory in our case).
+    // http://userguide.icu-project.org/design#TOC-ICU-Initialization-and-Termination
+    uloc_getDefault();
+#endif
   }
   
   void Toolbox::InitializeGlobalLocale(const char* locale)
@@ -1774,6 +1944,10 @@ namespace Orthanc
   void Toolbox::FinalizeGlobalLocale()
   {
     globalLocale_.reset();
+
+#if (ORTHANC_ENABLE_ICU == 1)
+    u_cleanup();
+#endif
   }
 
 
@@ -2526,7 +2700,6 @@ namespace Orthanc
 #endif
   }
 
-
   void Toolbox::RemoveSurroundingQuotes(std::string& value)
   {
     if (!value.empty() &&
@@ -2576,14 +2749,14 @@ namespace Orthanc
     return Toolbox::GetHumanTransferSpeed(full, sizeInBytes, GetElapsedNanoseconds());
   }
 
-  Toolbox::ElapsedTimeLogger::ElapsedTimeLogger(const std::string& message)
+  Toolbox::DebugElapsedTimeLogger::DebugElapsedTimeLogger(const std::string& message)
   : message_(message),
     logged_(false)
   {
     Restart();
   }
 
-  Toolbox::ElapsedTimeLogger::~ElapsedTimeLogger()
+  Toolbox::DebugElapsedTimeLogger::~DebugElapsedTimeLogger()
   {
     if (!logged_)
     {
@@ -2591,17 +2764,29 @@ namespace Orthanc
     }
   }
 
-  void Toolbox::ElapsedTimeLogger::Restart()
+  void Toolbox::DebugElapsedTimeLogger::Restart()
   {
     timer_.Restart();
   }
 
-  void Toolbox::ElapsedTimeLogger::StopAndLog()
+  void Toolbox::DebugElapsedTimeLogger::StopAndLog()
   {
     LOG(WARNING) << "ELAPSED TIMER: " << message_ << " (" << timer_.GetElapsedMicroseconds() << " us)";
     logged_ = true;
   }
 
+  Toolbox::ApiElapsedTimeLogger::ApiElapsedTimeLogger(const std::string& message) :
+    message_(message)
+  {
+    timer_.Restart();
+    CLOG(INFO, HTTP) << message_;
+  }
+  
+  Toolbox::ApiElapsedTimeLogger::~ApiElapsedTimeLogger()
+  {
+    CLOG(INFO, HTTP) << message_ << " (elapsed: " << timer_.GetElapsedMicroseconds() << " us)";
+  }
+
   std::string Toolbox::GetHumanFileSize(uint64_t sizeInBytes)
   {
     if (sizeInBytes < 1024)
